@@ -11,8 +11,9 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-from django.utils.translation import ugettext_lazy as _
 
+from django.core.urlresolvers import reverse
+from django.utils.translation import ugettext_lazy as _
 from horizon import exceptions
 from horizon import tables
 from openstack_dashboard.api import glance
@@ -22,10 +23,11 @@ class MarkImage(tables.LinkAction):
     name = "mark_image"
     verbose_name = _("Mark Image")
     url = "horizon:murano:images:mark_image"
-    classes = ("ajax-modal", "btn-create")
+    classes = ("ajax-modal",)
+    icon = "plus"
 
     def allowed(self, request, image):
-        return True
+        return request.user.is_superuser
 
 
 class RemoveImageMetadata(tables.DeleteAction):
@@ -34,10 +36,14 @@ class RemoveImageMetadata(tables.DeleteAction):
 
     def delete(self, request, obj_id):
         try:
-            glance.image_update(request, obj_id, properties={})
+            glance.image_update(request, obj_id,
+                                purge_props='murano_image_info')
         except Exception:
             exceptions.handle(request, _('Unable to remove metadata'),
-                              redirect='horizon:murano:images:index')
+                              redirect=reverse('horizon:murano:images:index'))
+
+    def allowed(self, request, image):
+        return request.user.is_superuser
 
 
 class MarkedImagesTable(tables.DataTable):
@@ -46,11 +52,15 @@ class MarkedImagesTable(tables.DataTable):
         link='horizon:project:images:images:detail',
         verbose_name=_('Image')
     )
-    type = tables.Column('type', verbose_name=_('Type'))
-    title = tables.Column('title', verbose_name=_('Title'))
+    type = tables.Column(lambda obj: getattr(obj, 'type', None),
+                         verbose_name=_('Type'))
+    title = tables.Column(lambda obj: getattr(obj, 'title', None),
+                          verbose_name=_('Title'),
+                          truncate=40)
 
-    class Meta:
+    class Meta(object):
         name = 'marked_images'
         verbose_name = _('Marked Images')
+        template = 'common/_data_table.html'
         table_actions = (MarkImage, RemoveImageMetadata)
         row_actions = (RemoveImageMetadata,)
